Assassin = {}

if select(2, UnitClass("player")) ~= "ROGUE" then
    DisableAddOn("Assassin")
    return
end

SLASH_Assassin1 = "/assassin"

local function InitializeVariables()
	Assassin.locked = Assassin.locked == nil and false or Assassin.locked
	Assassin.scale = Assassin.scale == nil and 1 or Assassin.scale
	Assassin.frequency = Assassin.frequency == nil and 0.05 or Assassin.frequency
	Assassin.latency = Assassin.latency == nil and 0.3 or Assassin.latency
	Assassin.previous = Assassin.previous == nil and true or Assassin.previous
	Assassin.cd = Assassin.cd == nil and true or Assassin.cd
	Assassin.bossonly = Assassin.bossonly == nil and false or Assassin.bossonly
	Assassin.misseffect = Assassin.misseffect == nil and true or Assassin.misseffect
	Assassin.glow = Assassin.glow == nil and true or Assassin.glow
	Assassin.step = Assassin.step == nil and true or Assassin.step
end

local events, targetHealthArray = {}, {}
local lastAbilityCalculation, GCD, currentTime, combatStartTime, energyToPool, T13SetPieces = 0, 0, 0, 0, 0, 0
local hostileTargetted, playerHasGlyphOfHemorrhage = false, false

local assassinPanel = CreateFrame("Frame", "assassinPanel", UIParent)
assassinPanel:SetPoint("CENTER", 0, -169)
assassinPanel:SetFrameStrata("BACKGROUND")
assassinPanel:Hide()
assassinPanel:RegisterForDrag("LeftButton")
assassinPanel:SetScript("OnDragStart", assassinPanel.StartMoving)
assassinPanel:SetScript("OnDragStop", assassinPanel.StopMovingOrSizing)
assassinPanel:SetMovable(true)
assassinPanel.icon = assassinPanel:CreateTexture(nil, "BACKGROUND")
assassinPanel.icon:SetAllPoints(assassinPanel)
assassinPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
assassinPanel.border = assassinPanel:CreateTexture(nil, "BORDER")
assassinPanel.border:SetAllPoints(assassinPanel)
assassinPanel.border:SetTexture("Interface\\AddOns\\Assassin\\border.blp")
assassinPanel.border:Hide()
local assassinPreviousPanel = CreateFrame("Frame", "assassinPreviousPanel", UIParent)
assassinPreviousPanel:SetPoint("BOTTOMRIGHT", assassinPanel, "BOTTOMLEFT", -10, -5)
assassinPreviousPanel:SetFrameStrata("BACKGROUND")
assassinPreviousPanel:Hide()
assassinPreviousPanel:RegisterForDrag("LeftButton")
assassinPreviousPanel:SetScript("OnDragStart", assassinPreviousPanel.StartMoving)
assassinPreviousPanel:SetScript("OnDragStop", assassinPreviousPanel.StopMovingOrSizing)
assassinPreviousPanel:SetMovable(true)
assassinPreviousPanel.icon = assassinPreviousPanel:CreateTexture(nil, "BACKGROUND")
assassinPreviousPanel.icon:SetAllPoints(assassinPreviousPanel)
assassinPreviousPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
assassinPreviousPanel.border = assassinPreviousPanel:CreateTexture(nil, "BORDER")
assassinPreviousPanel.border:SetAllPoints(assassinPreviousPanel)
assassinPreviousPanel.border:SetTexture("Interface\\AddOns\\Assassin\\border.blp")
local assassinCDPanel = CreateFrame("Frame", "assassinCDPanel", UIParent)
assassinCDPanel:SetPoint("BOTTOMLEFT", assassinPanel, "BOTTOMRIGHT", 10, -5)
assassinCDPanel:SetFrameStrata("BACKGROUND")
assassinCDPanel:Hide()
assassinCDPanel:RegisterForDrag("LeftButton")
assassinCDPanel:SetScript("OnDragStart", assassinCDPanel.StartMoving)
assassinCDPanel:SetScript("OnDragStop", assassinCDPanel.StopMovingOrSizing)
assassinCDPanel:SetMovable(true)
assassinCDPanel.icon = assassinCDPanel:CreateTexture(nil, "BACKGROUND")
assassinCDPanel.icon:SetAllPoints(assassinCDPanel)
assassinCDPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
assassinCDPanel.border = assassinCDPanel:CreateTexture(nil, "BORDER")
assassinCDPanel.border:SetAllPoints(assassinCDPanel)
assassinCDPanel.border:SetTexture("Interface\\AddOns\\Assassin\\border.blp")
local energyPoolText = assassinCDPanel:CreateFontString(nil, "OVERLAY")
energyPoolText:SetFont("Fonts\\FRIZQT__.TTF", 12, "")
energyPoolText:SetTextColor(1, 0.96, 0.41)
energyPoolText:SetShadowOffset(1, -1)
energyPoolText:SetShadowColor(0, 0, 0, 0.8)
energyPoolText:SetPoint("CENTER", assassinCDPanel, "CENTER", 0, -1)

local Ability = {}
Ability.__index = Ability

function Ability.add(spellId, buff, playerCast)
    local ability = {}
    setmetatable(ability, Ability)
	ability.spellId = spellId
    ability.buff = buff or false
    ability.playerCast = playerCast or false
    ability.name = GetSpellInfo(spellId)
    ability.icon = select(3, GetSpellInfo(spellId))
    return ability
end

function Ability:ready(seconds)
	return self:cooldown() <= (seconds or 0)
end

function Ability:getAura()
	for i = 1, 40 do
		if select(11, UnitAura(self.buff and "player" or "target", i, (self.buff and "HELPFUL" or "HARMFUL") .. (self.playerCast and "|PLAYER" or ""))) == self.spellId then
			return UnitAura(self.buff and "player" or "target", i, (self.buff and "HELPFUL" or "HARMFUL") .. (self.playerCast and "|PLAYER" or ""))
		end
	end
end

function Ability:remains()
	return max((select(7, self:getAura()) or 0) - currentTime - Assassin.latency, 0)
end

function Ability:up()
    return select(6, self:getAura()) == 0 or self:remains() > 0
end

function Ability:cooldown()
    local startTime, duration = GetSpellCooldown(self.spellId)
    return max(duration - (currentTime - startTime) - Assassin.latency, 0)
end

function Ability:stack()
    return select(4, self:getAura()) or 0
end

function Ability:known()
	return GetSpellBookItemInfo(self.name, "player") and true or false
end

function Ability:checkButtonAndGlow(button, on)
	if button then
		local actionType, spellIdOrMacro = GetActionInfo(button.action)
		if actionType then
			if (actionType == "spell" and spellIdOrMacro == self.spellId) or (actionType == "macro" and (select(2, GetMacroInfo(spellIdOrMacro))) == self.icon) then
				if on then
					ActionButton_ShowOverlayGlow(button)
				else
					ActionButton_HideOverlayGlow(button)
				end
			end
		end
	end
end

function Ability:glow(on)
	for i = 1, 12 do
		self:checkButtonAndGlow(_G["ActionButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarLeftButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarRightButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarBottomLeftButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarBottomRightButton" .. i], on)
	end
	if Dominos then
		for i = 1, 60 do self:checkButtonAndGlow(_G["DominosActionButton" .. i], on) end
	end
end

-- Rogue abilities
local TricksOfTheTrade = Ability.add(57934, true, true)
local SliceAndDice = Ability.add(5171, true, true)
local KillingSpree = Ability.add(51690, true, true)
local AdrenalineRush = Ability.add(13750, true, true)
local Eviscerate = Ability.add(2098, false, true)
local Rupture = Ability.add(1943, false, true)
local RevealingStrike = Ability.add(84617, false, true)
local SinisterStrike = Ability.add(1752, false, true)
local DeepInsight = Ability.add(84747, true, true)
local ShadowDance = Ability.add(51713, true, true)
local Stealth = Ability.add(1784, true, true)
local Vanish = Ability.add(1856, true, true)
local Vanished = Ability.add(11327, true, true)
local Shadowstep = Ability.add(36554, true, true)
local MasterOfSubtlety = Ability.add(31223, true, true)
local Mutilate = Ability.add(1329, false, true)
local Garrote = Ability.add(703, false, true)
local Vendetta = Ability.add(79140, false, true)
local Envenom = Ability.add(32645, true, true)
local FindWeakness = Ability.add(91023, false, true)
local Premeditation = Ability.add(14183, true, true)
local Ambush = Ability.add(8676, false, true)
local Preparation = Ability.add(14185, true, true)
local HemorrhageDoT = Ability.add(89775, false, true)
local HemorrhagePlayer = Ability.add(16511, false, true)
local Recuperate = Ability.add(73651, true, true)
local Backstab = Ability.add(53, false, true)
local FuryOfTheDestroyer = Ability.add(109949, true, true) -- Fangs of the Father (legendary daggers)
local Overkill = Ability.add(58427, true, true)
--   30% bleed damage
local MangleCat = Ability.add(33876, false, false)
local MangleBear = Ability.add(33878, false, false)
local Hemorrhage = Ability.add(16511, false, false)
local PetGore = Ability.add(35290, false, false)
local PetStampede = Ability.add(57386, false, false)
local PetTendonRip = Ability.add(50271, false, false)
local Trauma = Ability.add(46857, false, false)
-- Non-existent spell
local NoSpell = Ability.add(0, false, false)
local PoolSpell = Ability.add(0, false, false)

local abilityNow, oldAbilityNow, abilityPrevious, abilityCD = NoSpell, NoSpell, NoSpell, NoSpell
PoolSpell.icon = "Interface\\AddOns\\Assassin\\PoolSpell.blp"

local function ComboPoints()
    return GetComboPoints("player", "target")
end

local function Energy()
    return min(UnitPower("player") + (GetPowerRegen() * Assassin.latency), UnitPowerMax("player")) or UnitPowerMax("player")
end

local function TargetHealthPercentage()
    return UnitHealth("target") / UnitHealthMax("target") * 100
end

local function TargetTimeToDie()
    local targetTimeToDie = UnitHealth("target") / ((targetHealthArray[1] - targetHealthArray[#targetHealthArray]) / 3) - Assassin.latency
    return targetTimeToDie >= 0 and targetTimeToDie or 600
end

local function TargetIsBoss()
	return UnitLevel("target") == -1 or (UnitLevel("target") >= UnitLevel("player") + 2 and not UnitInRaid("player"))
end

local function TimeInCombat()
    return currentTime - (combatStartTime > 0 and combatStartTime or currentTime) + Assassin.latency
end

local function BleedDebuffActive()
	return MangleCat:up() or MangleBear:up() or Hemorrhage:up() or PetGore:up() or PetStampede:up() or PetTendonRip:up() or Trauma:up()
end

local function Stealthed()
	return Stealth:up() or Vanished:up() or ShadowDance:up()
end

local function PlayerInGroup()
	return UnitInParty("party1")
end

local function PoolEnergy(amount)
	energyPoolText:SetText(amount == 0 and "" or max(floor(amount - Energy()), 0))
	return PoolSpell
end

local function CalculateBestChoiceAbilityAssassination()
	if Assassin.cd and (not Assassin.bossonly or TargetIsBoss()) then
		if Vendetta:ready() and SliceAndDice:up() then
			abilityCD = Vendetta
		elseif Vanish:ready() and not Overkill:up() and Energy() > 45 and Energy() < 95 then
			abilityCD = Vanish
		elseif TricksOfTheTrade:ready() and PlayerInGroup() and Energy() < 70 then
			abilityCD = TricksOfTheTrade
		else
			abilityCD = NoSpell
		end
	else
		abilityCD = NoSpell
	end
	if not InCombatLockdown() and not Stealth:up() then
		return Stealth
	elseif Stealth:up() then
		return Garrote
	elseif not SliceAndDice:up() and ComboPoints() ~= 0 then
		return SliceAndDice
	elseif Rupture:remains() < 2 and ComboPoints() ~= 0 and (TimeInCombat() < 6 or SliceAndDice:remains() > 6) then
		return Rupture
	elseif (ComboPoints() >= 4 and (not Envenom:up() or Energy() > 75 or FuryOfTheDestroyer:up())) or (ComboPoints() >= 2 and SliceAndDice:remains() < 3) then
		return Envenom
	elseif ComboPoints() < 5 and TargetHealthPercentage() < 35 then
		return Backstab
	elseif ComboPoints() < 4 and TargetHealthPercentage() >= 35 then
		return Mutilate
	else
		return NoSpell
	end
end

local function CalculateBestChoiceAbilityCombat()
	if Assassin.cd and (not Assassin.bossonly or TargetIsBoss()) then
		if KillingSpree:ready() and (((Energy() < 50 or ComboPoints() == 5) and not AdrenalineRush:up() and SliceAndDice:remains() > 4) or (DeepInsight:up() and DeepInsight:remains() < 3) or TargetTimeToDie() < 6) and not FuryOfTheDestroyer:up() then
			abilityCD = KillingSpree
		elseif AdrenalineRush:ready() and (((Energy() < 50 or ComboPoints() == 5) and not KillingSpree:ready(8)) or TargetTimeToDie() < 16) then
			abilityCD = AdrenalineRush
		elseif TricksOfTheTrade:ready() and PlayerInGroup() and ((AdrenalineRush:up() and Energy() < 40) or Energy() < 70) then
			abilityCD = TricksOfTheTrade
		else
			abilityCD = NoSpell
		end
	else
		abilityCD = NoSpell
	end
	if SliceAndDice:remains() < 2 and ComboPoints() ~= 0 then
		return SliceAndDice
	elseif ComboPoints() == 5 and DeepInsight:up() then
		return Eviscerate
	elseif not Rupture:up() and ComboPoints() == 5 and BleedDebuffActive() and TargetTimeToDie() > 10 then
		return Rupture
	elseif ComboPoints() == 5 then
		return Eviscerate
	elseif ComboPoints() == 4 and not RevealingStrike:up() then
		return RevealingStrike
	elseif ComboPoints() ~= 5 then
		return SinisterStrike
	end
end

local function CalculateBestChoiceAbilitySubtlety()
	if Assassin.cd and (not Assassin.bossonly or TargetIsBoss()) then
		if ShadowDance:ready() and ComboPoints() ~= 5 and InCombatLockdown() and not Stealth:up() then
			if Energy() > 75 then
				PoolEnergy(0)
				abilityCD = ShadowDance
			else
				abilityCD = PoolEnergy(75)
			end
		elseif Vanish:ready() and (Shadowstep:ready() or not Assassin.step) and ComboPoints() <= 1 and TimeInCombat() > 10 and not ShadowDance:up() and not MasterOfSubtlety:up() and not FindWeakness:up() then
			if Energy() > 50 then
				PoolEnergy(0)
				abilityCD = Vanish
			else
				abilityCD = PoolEnergy(50)
			end
		elseif Preparation:ready() and Energy() < 50 and not Vanish:ready(60) then
			abilityCD = Preparation
		elseif TricksOfTheTrade:ready() and PlayerInGroup() and Energy() < 70 then
			abilityCD = TricksOfTheTrade
		else
			abilityCD = NoSpell
		end
	else
		abilityCD = NoSpell
	end
	if not InCombatLockdown() and not Stealth:up() then
		return Stealth
	elseif not InCombatLockdown() and SliceAndDice:remains() < 6 and ComboPoints() == 5 then
		return SliceAndDice
	elseif not InCombatLockdown() and Recuperate:remains() < 6 and ComboPoints() == 5 then
		return Recuperate
	elseif Assassin.step and Shadowstep:ready() and Stealthed() then
		return Shadowstep
	elseif Premeditation:ready() and Stealthed() and ComboPoints() <= 3 then
		return Premeditation
	elseif Stealthed() and ComboPoints() <= 4 then
		return Ambush
	elseif SliceAndDice:remains() < 3 and ComboPoints() == 5 then
		return SliceAndDice
	elseif not Rupture:up() and ComboPoints() == 5 then
		return Rupture
	elseif Recuperate:remains() < 3 and ComboPoints() == 5 then
		return Recuperate
	elseif Rupture:remains() > 1 and ComboPoints() == 5 then
		return Eviscerate
	elseif ((playerHasGlyphOfHemorrhage and (HemorrhageDoT:remains() < 4 and HemorrhagePlayer:remains() < 58)) or not BleedDebuffActive()) and (ComboPoints() <= 3 or (ComboPoints() <= 4 and Energy() > 70)) then
		return Hemorrhage
	elseif ComboPoints() <= 3 or (ComboPoints() <= 4 and Energy() > 70) then
		return Backstab
	else
		return NoSpell
	end
end

local function CalculateBestChoiceAbility()
	if Mutilate:known() then
		return CalculateBestChoiceAbilityAssassination()
	elseif KillingSpree:known() then
		return CalculateBestChoiceAbilityCombat()
	elseif Shadowstep:known() then
		return CalculateBestChoiceAbilitySubtlety()
	else
		return NoSpell
	end
end

function events:PLAYER_EQUIPMENT_CHANGED()
	T13SetPieces = (GetItemInfo(GetInventoryItemID("player", 1) or 0) == "Blackfang Battleweave Helmet" and 1 or 0) +
		(GetItemInfo(GetInventoryItemID("player", 10) or 0) == "Blackfang Battleweave Gloves" and 1 or 0) +
		(GetItemInfo(GetInventoryItemID("player", 5) or 0) == "Blackfang Battleweave Tunic" and 1 or 0) +
		(GetItemInfo(GetInventoryItemID("player", 3) or 0) == "Blackfang Battleweave Spaulders" and 1 or 0) +
		(GetItemInfo(GetInventoryItemID("player", 7) or 0) == "Blackfang Battleweave Legguards" and 1 or 0)
end

function events:PLAYER_TARGET_CHANGED()
	hostileTargetted = not UnitCanAttack("player", "target")
end

function events:ACTIVE_TALENT_GROUP_CHANGED()
	playerHasGlyphOfHemorrhage = false
	for i = 1, GetNumGlyphSockets() do
		if select(4, GetGlyphSocketInfo(i)) == 56807 then
			playerHasGlyphOfHemorrhage = true
			break
		end
	end
end

function events:PLAYER_ENTERING_WORLD()
	events:ACTIVE_TALENT_GROUP_CHANGED()
	events:PLAYER_EQUIPMENT_CHANGED()
end

function events:ADDON_LOADED(name)
	if name == "Assassin" then
		if not Assassin.scale then
			print("It looks like this is your first time running Assassin, why don't you take some time to familiarize yourself with the commands?")
			print("Type |cFFFFD000/assassin|r for a list of commands.")
		end
		InitializeVariables()
		assassinPanel:SetSize(64, 64)
		assassinPanel:SetScale(Assassin.scale)
		assassinPanel:EnableMouse(not Assassin.locked)
		assassinPreviousPanel:SetSize(42, 42)
		assassinPreviousPanel:SetScale(Assassin.scale)
		assassinPreviousPanel:EnableMouse(not Assassin.locked)
		assassinCDPanel:SetSize(42, 42)
		assassinCDPanel:SetScale(Assassin.scale)
		assassinCDPanel:EnableMouse(not Assassin.locked)
        for i = 1, math.floor(3 / Assassin.frequency) do targetHealthArray[i] = 0 end
		assassinPanel:Show()
	end
end

function events:UNIT_SPELLCAST_SENT(srcName, spellName)
	if Assassin.previous and srcName == "player" and spellName == abilityNow.name then
		abilityPrevious = abilityNow
	end
end

function events:PLAYER_REGEN_DISABLED()
    combatStartTime = GetTime()
end

function events:PLAYER_REGEN_ENABLED()
    combatStartTime = 0
end

function events:COMBAT_LOG_EVENT_UNFILTERED(self, eventType, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, spellId, spellName, spellSchool, auraType)
	if Assassin.previous then
		if eventType == "SPELL_CAST_SUCCESS" and srcGUID == UnitGUID("player") and spellId == abilityPrevious.spellId then
			assassinPreviousPanel.icon:SetTexture(abilityPrevious.icon)
			assassinPreviousPanel.border:SetTexture("Interface\\AddOns\\Assassin\\border.blp")
			assassinPreviousPanel:Show()
		elseif Assassin.misseffect and eventType == "SPELL_MISSED" and srcGUID == UnitGUID("player") and spellId == abilityPrevious.spellId then
			assassinPreviousPanel.border:SetTexture("Interface\\AddOns\\Assassin\\misseffect.blp")
		end
	end
end

assassinPanel:SetScript("OnUpdate", function(self, elapsed)
    lastAbilityCalculation = lastAbilityCalculation + elapsed;
    if lastAbilityCalculation >= Assassin.frequency then
		if UnitCanAttack("player", "target") and not UnitInVehicle("player") then
			if not hostileTargetted then
				assassinPanel:EnableMouse(not Assassin.locked)
				assassinPreviousPanel:Hide()
				assassinCDPanel:Hide()
				for i = 1, #targetHealthArray do targetHealthArray[i] = UnitHealth("target") end
				hostileTargetted = true
			else
				targetHealthArray[#targetHealthArray + 1] = UnitHealth("target")
				table.remove(targetHealthArray, 1)
			end
			oldAbilityNow = abilityNow
			oldAbilityCD = abilityCD
			GCD = 1.5 - 1.5 * (GetCombatRatingBonus(20) * 0.01)
			currentTime = GetTime()
			abilityNow = CalculateBestChoiceAbility()
			if Assassin.glow then
				abilityNow:glow(true)
				abilityCD:glow(true)
			end
			if abilityNow ~= oldAbilityNow then
				assassinPanel.icon:SetTexture(abilityNow.icon)
				if abilityNow.icon then
					assassinPanel.icon:Show()
					assassinPanel.border:Show()
				else
					assassinPanel.icon:Hide()
					assassinPanel.border:Hide()
				end
				if Assassin.glow then
					oldAbilityNow:glow(false)
				end
			end
			if abilityCD ~= oldAbilityCD then
				assassinCDPanel.icon:SetTexture(abilityCD.icon)
				if abilityCD.icon then
					assassinCDPanel:Show()
				else
					assassinCDPanel:Hide()
				end
				if Assassin.glow then
					oldAbilityCD:glow(false)
				end
			end
		elseif hostileTargetted then
			if Assassin.glow then
				abilityNow:glow(false)
				abilityCD:glow(false)
			end
			abilityNow = NoSpell
			abilityCD = NoSpell
			assassinPanel.icon:Hide()
			assassinPanel.border:Hide()
			assassinPanel:EnableMouse(false)
			assassinPreviousPanel:Hide()
			assassinCDPanel:Hide()
			for i = 1, #targetHealthArray do targetHealthArray[i] = 0 end
			hostileTargetted = false
		end
        lastAbilityCalculation = 0
    end
end)

assassinPanel:SetScript("OnEvent", function(self, event, ...) events[event](self, ...) end)
for k in pairs(events) do
    assassinPanel:RegisterEvent(k)
end

function SlashCmdList.Assassin(msg, editbox)
    msg = { strsplit(" ", strlower(msg)) }
    if msg[1] == "locked" then
        if msg[2] then
			Assassin.locked = msg[2] == "on" 
			assassinPanel:EnableMouse(not Assassin.locked and assassinPanel.icon:IsVisible())
			assassinPreviousPanel:EnableMouse(not Assassin.locked)
			assassinCDPanel:EnableMouse(not Assassin.locked)
		end
        print("Assassin - Locked: " .. (Assassin.locked and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "scale" then
        if msg[2] then
			Assassin.scale = tonumber(msg[2]) or 1
			assassinPanel:SetScale(Assassin.scale)
			assassinPreviousPanel:SetScale(Assassin.scale)
			assassinCDPanel:SetScale(Assassin.scale)
		end
		print("Assassin - Scale: |cFFFFD000" .. Assassin.scale .. "|r times")
	elseif msg[1] == "frequency" then
		Assassin.frequency = tonumber(msg[2]) or 0.05
		print("Assassin - Calculation frequency: Every |cFFFFD000" .. Assassin.frequency .. "|r seconds")
	elseif msg[1] == "latency" then
		Assassin.latency = tonumber(msg[2]) or 0.3
		print("Assassin - Player reaction time: |cFFFFD000" .. Assassin.latency .. "|r seconds")
    elseif msg[1] == "glow" then
        if msg[2] then
			Assassin.glow = msg[2] == "on"
			if Assassin.glow then
				abilityNow:glow(true)
				abilityCD:glow(true)
			else
				abilityNow:glow(false)
				abilityCD:glow(false)
			end
		end
        print("Assassin - Glowing ability buttons: " .. (Assassin.glow and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "previous" then
        if msg[2] then Assassin.previous = msg[2] == "on" end
        print("Assassin - Previous ability icon: " .. (Assassin.previous and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "cd" then
        if msg[2] then Assassin.cd = msg[2] == "on" end
        print("Assassin - Cooldown ability icon: " .. (Assassin.cd and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "misseffect" then
        if msg[2] then Assassin.misseffect = msg[2] == "on" end
        print("Assassin - Red border around previous ability when it fails to hit: " .. (Assassin.misseffect and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "step" then
        if msg[2] then Assassin.step = msg[2] == "on" end
        print("Assassin - Shadowstep: " .. (Assassin.step and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "bossonly" then
        if msg[2] then Assassin.bossonly = msg[2] == "on" end
        print("Assassin - Only use cooldowns on bosses: " .. (Assassin.bossonly and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "reset" then
		assassinPanel:ClearAllPoints()
        assassinPanel:SetPoint("CENTER", 0, -169)
		assassinPreviousPanel:ClearAllPoints()
		assassinPreviousPanel:SetPoint("BOTTOMRIGHT", assassinPanel, "BOTTOMLEFT", -10, -5)
		assassinCDPanel:ClearAllPoints()
		assassinCDPanel:SetPoint("BOTTOMLEFT", assassinPanel, "BOTTOMRIGHT", 10, -5)
		print("Assassin - Position has been reset to default")
    else
        print("Assassin (version: |cFFFFD000" .. GetAddOnMetadata("Assassin", "Version") .. "|r) - Commands:")
        print("  /assassin locked |cFF00C000on|r/|cFFC00000off|r - lock the Assassin UI so that it can't be moved")
		print("  /assassin scale |cFFFFD000[number]|r - set the scale of the Assassin UI (default is 1 times)")
		print("  /assassin frequency |cFFFFD000[number]|r - set the calculation frequency (default is every 0.05 seconds)")
		print("  /assassin latency |cFFFFD000[number]|r - set the player's reaction time (default is 0.3 seconds)")
		print("  /assassin glow |cFF00C000on|r/|cFFC00000off|r - glowing ability buttons on action bars")
		print("  /assassin previous |cFF00C000on|r/|cFFC00000off|r - previous ability icon")
		print("  /assassin cd |cFF00C000on|r/|cFFC00000off|r - cooldown ability icon (turn off for manual cooldown management)")
		print("  /assassin misseffect |cFF00C000on|r/|cFFC00000off|r - red border around previous ability when it fails to hit")
		print("  /assassin step |cFF00C000on|r/|cFFC00000off|r - use Shadowstep (turn off for some bosses)")
		print("  /assassin bossonly |cFF00C000on|r/|cFFC00000off|r - only use cooldowns on bosses")
        print("  /assassin reset - reset the location of the Assassin UI to default")
		if Claw_Resources then
			print("For Claw Resources commands, please type |cFFFFD000/cres|r.")
		end
    end
end
